#include "kynetloadratethread.h"

#include <QDebug>
#define UPDATE_LOADRATE_TIMER 1000

KyNetLoadRateThread::KyNetLoadRateThread(QString dev, QObject *parent)
    :m_deviceName(dev), QThread(parent)
{

}

KyNetLoadRateThread::~KyNetLoadRateThread()
{

}

void KyNetLoadRateThread::run()
{
    m_updateRatesTimer = new QTimer(this);
    m_updateRatesTimer->setTimerType(Qt::PreciseTimer);
    connect(m_updateRatesTimer, &QTimer::timeout,  [&]() {
        updateLoadRate();
    });

    m_updateRatesTimer->start(UPDATE_LOADRATE_TIMER);
    exec();
}

void KyNetLoadRateThread::updateLoadRate()
{
    if (getCurrentLoadRate(m_deviceName, &m_startRcvRate, &m_startTxRate) == -1) {
        m_startRcvRate = m_endRcvRate;
        return;
    }

    m_deltaRcv = (m_startRcvRate - m_endRcvRate) / 1024;
    m_deltaTx = (m_startTxRate - m_endTxRate) / 1024;

    //简易滤波
    if (m_deltaRcv < 0 || m_deltaTx < 0) {
        m_deltaRcv = 0;
        m_deltaTx = 0;
    }
    else if (m_endRcvRate == 0 || m_endTxRate == 0){
        m_deltaRcv = 0;
        m_deltaTx = 0;
    }

    m_endRcvRate = m_startRcvRate;
    m_endTxRate = m_startTxRate;
}

int KyNetLoadRateThread::getCurrentLoadRate(QString dev, long *saveRate, long *txRate)
{
    FILE * netDevFile; //文件指针
    char buffer[1024]; //文件中的内容暂存在字符缓冲区里
    //size_t bytes_read; //实际读取的内容大小
    char * match; //用以保存所匹配字符串及之后的内容
    char * device;//将QString转为Char *
    QByteArray ba = dev.toLatin1(); // must
    device = ba.data();
    int counter = 0;
    char tmpValue[128];

    if ((NULL == device) || (NULL == saveRate) || (NULL == txRate)) {
        qDebug() << "LOG_FLAG" << "parameter pass error" ;
        return -1;
    }

    if ((netDevFile = fopen("/proc/net/dev", "r")) == NULL) {
        //打开文件/pro/net/dev/，从中读取流量数据
        qDebug() << "LOG_FLAG" << "error occurred when try to open file /proc/net/dev/";
        return -1;
    }
    memset(buffer, 0, sizeof(buffer));

    while (fgets(buffer, sizeof(buffer), netDevFile) != NULL) {
        match = strstr(buffer, device);

        if (NULL == match) {
            // qDebug()<<"No eth0 keyword to find!";
            continue;
        } else {
            match = match + strlen(device) + strlen(":"); //地址偏移到冒号
            sscanf(match, "%ld ", saveRate);
            memset(tmpValue, 0, sizeof(tmpValue));
            sscanf(match, "%s ", tmpValue);
            match = match + strlen(tmpValue);
            for (size_t i = 0; i < strlen(buffer); ++i) {
                if (0x20 == *match) {
                    match++;
                } else {
                    if (8 == counter) {
                        sscanf(match, "%ld ", txRate);
                    }
                    memset(tmpValue, 0, sizeof(tmpValue));
                    sscanf(match, "%s ", tmpValue);
                    match = match + strlen(tmpValue);
                    counter++;
                }
            }
        }
    }

    fclose(netDevFile);
    netDevFile = nullptr;

    return 0; //返回成功
}
